function bayes
%bayes performs a Bayesian estimation of the  empirical  model with permanent and temporary monetary and nonmonetary shocks, studied in Uribe, Martn, ``The Neo-Fisher Effect: Econometric Evidence from Empirical and Optimizing Models.''
%param_chain is a matrix containing draws from the posterior distribution of the vector of estimated parameters
%mean_param is the  mean of param_chain
%posterior_chain  evaluates the posterior density (up to a scalar that depends on the data) at  param_chain
%likelihood_chain  evaluates the likelihood function   at  param_chain.
%varcov_param variance/covariance matrix of the vector of estimated parameters, computed using the MCMC chain. 
%var_param is a vector equal to the diagonal of  varcov_param. 
%std_param is a vector equal to the square root of var_param
%Ci lower choleski decomposition of the  variance-covariance matrix of the stand-in (or proposal) distribution. 
%accept_rate fraction of proposal draws accepted. 
% Martn Uribe, 06-Mar-2017.


load clone.mat %initial  value of parameter vector. Produced by running run_bayes.m

format compact

ndraw = 1e6; %number of draws of MCMC  chain 
nburn = 1e5; %burn-in draws
ntrain = 1e5;
ntotal = ndraw + nburn+ntrain; 

%read data
date_end = 2018.25; %last observation
[Dy,Dpai,Dff, y, pai,ff,date, readme_data] = read_data(date_end);
r = ff-pai;
Y = [Dy r Dff];
X = 0*Dy+1; %intercept
T = size(Y,1);
meanY = mean(Y)';
stdY = std(Y)';
varY = var(Y)';

create_structure; %creaes the matrices that constitute the structure of the SVAR;

load clone.mat MPE 
%produced by running clone.m in  z:\uribe\fisher\us\2008 
%MPE serves as the starting parameter value

param = MPE; %parameter draw 
likelihood = f(param,Y,X); %draw of log posterior likelihood 
%evaluate the prior likelihood
prior = get_prior(param,iB,iC,iRHO,iP,iAp,iR,meanY,varY,stdY,T,iBown,iBother);
posterior = likelihood+prior;%draw of log posterior likelihood.

np = length(param); %# of estimated parameters

%Initialize chains:
param_chain = zeros(np,ndraw); %Matrix collecting the history of posterior draws

posterior_chain = zeros(ndraw,1); %Vector collecting the history of evaluations of the log posterior function

likelihood_chain = zeros(ndraw,1); %Vector collecting the history of evaluations of the log posterior function
accept_chain =  zeros(ndraw,1);
accept_rate = 0;

%The following five objects are used to compute the variance-covariance matrix of the proposal distribution in an adaptive fashion
Mparam = 0; %weighted average of parameter draws, with weight lambda, given below
MSparam = 0; %weighted average of square parameters using the weight lambda. 
Macc = 0.23; %weighted  average of accept rate using weight lambda
V = eye(np); %variance-covariance matrix of proposal distribution
Ci = eye(np);  %Cholesky decomposition of V, V=Ci*Ci';

tar = 0.15; %target for acceptance rate
kappa = 0.001; %speed of adjustment of scaler of proposal distribution 
lambda = (1/2)^1e-4; %weight on current draw in computing  Mparam, MSparam, and Macc. This value puts a cumulative weight of 1/2 to the last 1e4 draws. 
n = 1; %counter number of iterations
nn = 1; %starts counting when n>nburn+ntrain

t1 = tic; %starting time 

%start drawing from posterior distribution
while n<= ntotal

%update the variance-covariance matrix of the proposal distribution 
if mod(n,1000)==0
Vrecent = MSparam-Mparam*Mparam';
try
V = (V+Vrecent)/2;
Ci= chol(V,'lower');
end %try
end %if mod(n,..

%Draw parameter value
tryparam = param + kappa*Ci * randn(np,1);
if (n<ntrain) & (rand<0.1)
tryparam = MPE;
end
trylikelihood = -inf;
exitflag = 0;

tryprior = get_prior(tryparam,iB,iC,iRHO,iP,iAp,iR,meanY,varY,stdY,T,iBown,iBother);

if tryprior>-inf  %%Make sure the draw is within bounds

evaluate_structure; %assign values to the matrices of the SVAR

if unit_circle==1 %Check that the  SVAR system produces stable dynamics
try
trylikelihood =  log_like_kalman(Y,F,Q,Hp',R,X,Ap');
end %try
end %if unit_circle

if ~isreal(trylikelihood) | isnan(trylikelihood) | ~isfinite(trylikelihood)
trylikelihood = -inf;
end

tryposterior =trylikelihood + tryprior;

%Determine whether to accept or reject the draw
if  rand <= exp(tryposterior-posterior) 

exitflag = 5;  %indicator of draw accepted

param = tryparam;
posterior = tryposterior;
likelihood = trylikelihood;
prior = tryprior;
end %if rand

end %if tryprior>-inf 

%Update the weighted averages of the parameter draws, of their squares. and of the acceptance rate
Mparam = Mparam * lambda + param * (1-lambda);
MSparam = MSparam * lambda  + param*param' * (1-lambda);
Macc = Macc * lambda + (exitflag==5) * (1-lambda);

%Update the scaler of the proposal distribution
kappa + 0.001*sign(Macc - tar);
kappa = ans *(ans>0) + 0.9*kappa * (ans<=0);

%Update the MCMC chains
if n>nburn+ntrain
param_chain(:,nn) = param;
posterior_chain(nn) = posterior;
likelihood_chain(nn) = likelihood;
accept_chain(nn) = (exitflag==5);

if rand<10/(tar*ntotal);
disp(['n = ' num2str(n) ]);
disp([num2str(round(n/ntotal*100)) '% completed. Time remaining ' num2str(round(toc(t1)/n*(ntotal-n)/60^2*10)/10)  ' hours.' ]);
disp('  ')
end

nn = nn+1;
end

accept_rate = (n-1)/n * accept_rate + 1/n * (exitflag==5);

n = n+1;

end %while n<ntotal

%Calculate some first and second moments from chains
mean_param = mean(param_chain,2);
varcov_param = cov(param_chain');
var_param = diag(varcov_param);
std_param = sqrt(var_param);
median_param = median(param_chain,2);
[mp,j] = max(posterior_chain);
mpe = param_chain(:,j); %model of posterior distribution
accept_rate = mean(accept_chain);

%evaluate matrices of the SVAR at the mean parameter value
tryparam = mean_param;
evaluate_structure

save bayes.mat
